/*
 * This module defines methods/functions to handle events that are also registered
 * for Google Smart Shopping here.
 *
 * @Param:
 * - gtagConfig - configuration object from the published GSS body script
 *                (DE_EPAGES/GoogleSmartShopping/Templates/SF/SF.Body-Script-GoogleSmartShopping.html)
 *                Structure:
 *                {
 *                    gtag: gtag,
 *                    availableCountries: availableCountries,
 *                    googlePortalsLocale: googlePortalsLocale
 *                }
 *
 * Description/Structure of objects:
 *
 * product: {
 *     availabilityText: "Available"
 *     available: true
 *     basePrice: {refQuantity: {…}, refPrice: {…}, formatted: "1 m³ = €0,12", quantity: {…}}
 *     bulkPrices: null
 *     conditionMicrodata: "NewCondition"
 *     customAttributes: [{…}]
 *     deliveryPeriod: "2-3"
 *     deliveryPeriodUnit: "DAYS"
 *     description: null
 *     energyLabel: null
 *     energyLabelSourceFile: null
 *     gtin: 7501054530107
 *     hasCrossSelling: false
 *     hasStockLevel: true
 *     hasVariations: false
 *     href: "/p/homemade-cherry-jam"
 *     image: null
 *     isStrikePriceRRP: false
 *     isVariationMaster: false
 *     isVariationProduct: false
 *     isVisible: true
 *     links: (5) [{…}, {…}, {…}, {…}, {…}]
 *     listPrice: null // deprecated, use strikePrice and isStrikePriceRRP instead
 *     lowestPrice: null
 *     mainCategoryId: "5954B711-E377-2A90-C400-D5809AB3F62B"
 *     manufacturer: null
 *     manufacturerPrice: null // deprecated, use strikePrice and isStrikePriceRRP instead
 *     metaDescription: ""
 *     name: "Homemade Cherry Jam"
 *     onStock: true
 *     outOfStock: false
 *     pickupPeriod: ""
 *     price: {taxType: "NET", formatted: "€40,00", amount: 40, currency: "EUR"}
 *     productDataSheet: null
 *     productId: "5954B706-E701-F357-A52D-D5809AB3F606"
 *     productVariationSelection: null
 *     productVariationValues: ""
 *     sku: "1007"
 *     slideshow: [{…}, {…}, {…}, {…}]
 *     slug: "homemade-cherry-jam"
 *     stockLevelClass: "in"
 *     stockLevelMicrodata: "InStock"
 *     strikePrice: null
 *     title: "Homemade Cherry Jam"
 *     variationMaster: null
 *     variations: null
 *     vatNote: "components.productComponent.priceExclusiveVat"
 *     warnStock: false
 *     weight: null
 * }
 *
 * cart: {
 *     billingAddress: null
 *     canHaveCoupon: true
 *     cartId: "5C06901F-150A-3D9B-B144-D509AB34800"
 *     checkoutButtons: [{…}]
 *     checkoutState: null
 *     checkoutUrl: "https://pm.epages.com/epages/apidocu.sf/?ObjectPath=/Shops/apidocu/AnonymousUsers/1/Baskets/46072&ChangeAction=PickupBasketPickupToken=MjNiZThlOGM0MjAzZWQ0N2ZjYmZmZDFiZjI3OTQxMzkwOWY4ZjZlOGE4NTM0ZWIxMjg3NmY2NzBiY2IxYzQ1OV8xNDgyMjIzMDcz"
 *     coupon: null
 *     couponCampaign: null
 *     customerEmail: null
 *     grandAmount: "76,01 €"
 *     grandAmountNote: "components.productComponent.priceExclusiveVat"
 *     grandTotal: {amount: 76.01, currency: "EUR", formatted: "€76,01"}
 *     minimumOrderValue: null
 *     mustAcceptTermsAndConditions: false
 *     netAmount: "76,01 €"
 *     paymentLineItem: {lineItemPrice: {…}, paymentMethod: {…}}
 *     pickupToken: "ZjM3M2Q2YmY4jkFjYWRlZTIzZTBlYzQwMDU4MjYzZjYwNDNhZGY0NWM1N2JiNjZhMGI0YWNlNWFkYzU4ZTQ3OF8xNTQzOTM5MjIx"
 *     potentialBasketDiscounts: null
 *     productLineItems: [{…}, {…}, {…}]
 *     registerSessionUrl: "https://pm.epages.com/epages/apidocu.sf/?ObjectPath=/Shops/apidocu/AnonymousUsers/2/Baskets/46072&ChangeAction=PickupBasket&PickupToken=NTRjZWJmNjdhNzNlYTUzNDAxZTgyZTc4ODYwYTliMDUxMDIzNDQ2OWY2NWQ5NWRmN2Q2YmVjZjVjNzljOTQ2N18xNDgyMjI3OTE3"
 *     shippingAddress: null
 *     shippingLineItem: {lineItemPrice: {…}, shippingMethod: {…}}
 *     status: null
 *     subAmount: "76,01 €"
 *     taxType: "NET"
 *     taxes: []
 *     totalBasketDiscount: null
 *     totalNumberOfItems: 3
 *     _links: null
 * }

 * NOTE:
 * For development some logs can be activated. For this add "&SetDebug=1" to the URL, open the web console
 * and reload the page.

 */

define('de_epages/googlesmartshopping/googlesmartshopping-controller', [
    'jquery',
    '$pub!__gtag__',

    'jquery/i18n'
], function($, gtagConfig) {
	var settings,

        /*
         * Handling call to action (CTA) events like *click* on products
         */
        onProductCTAEvent = function(event) {
            var product = event.detail.product,
                googleProduct,
                eventParameters;

            if(!product || !product.price || product.price.amount <= 0) {
              return;
            }

            // parse relevant product data
            googleProduct = getGoogleProduct(product);

            // generate event parameters
            eventParameters = {
              'send_to': settings.adsCampaignConversionId,
              'ecomm_pagetype': 'product',
              'value': googleProduct.price,
              'items': [googleProduct]
            };

            settings.debug && console.log('de_epages/googlesmartshopping/googlesmartshopping-controller - onProductCTAEvent: ', product, eventParameters);
            // tag google event
            gtagConfig.gtag('event', 'view_item', eventParameters);
        },

        /*
         * Handling view events for single products
         */
        onSingleProductViewEvent = function(event) {
            onProductCTAEvent(event);
        },

        /*
         * Handling view events for product lists
         */
        onProductListViewEvent = function(event) {
            var products = event.detail.products,
                data = event.detail.data,
                googleProducts = [],
                eventParameters;

            if(!products || products.length <= 0) {
              return;
            }

            // parse relevant product data for each product in the current view
            products.forEach(function(product) {
                googleProducts.push(getGoogleProduct(product));
            });

            // generate event parameters
            eventParameters = {
              'send_to': settings.adsCampaignConversionId,
              'ecomm_pagetype': 'product',
              'item_list_name': data && data.list ? data.list : "default_list",
              'items': [googleProducts]
            };

            settings.debug && console.log('de_epages/googlesmartshopping/googlesmartshopping-controller - onProductListViewEvent: ', products, eventParameters);
            // tag google event
            gtagConfig.gtag('event', 'view_item_list', eventParameters);
        },

        /*
         * Handling event if a product is added to the cart
         */
        onCartAddEvent = function(event) {
            var product = event.detail.product,
                eventQuantity = event.detail.quantity,
                googleProduct,
                eventParameters;

            if(!product || !product.price || product.price.amount <= 0 || eventQuantity <= 0) {
              return;
            }

            // parse relevant product data
            googleProduct = getGoogleProduct(product);
            // get quantity from the triggering event
            googleProduct.quantity = eventQuantity;

            // generate event parameters
            eventParameters = {
              'send_to': settings.adsCampaignConversionId,
              'ecomm_pagetype': 'cart',
              // 'value': Number((googleProduct.price * eventQuantity).toFixed(2)),
              'value': $.i18n.parseNumber((googleProduct.price * eventQuantity).toFixed(2), 10, {
                    region: $.i18n.settings.region
              }),
              'items': [googleProduct]
            };

            settings.debug && console.log('de_epages/googlesmartshopping/googlesmartshopping-controller - onCartAddEvent: ', product, eventQuantity, eventParameters);
            // tag google event
            gtagConfig.gtag('event', 'add_to_cart', eventParameters);
        },

        /*
         * Handling event if cart is viewed
         */
        onCartViewEvent = function(event) {
            var cart = event.detail.cart,
                data = event.detail.data,
                googleProducts,
                eventParameters;

            if(!cart || !cart.productLineItems || cart.grandTotal.amount <= 0) {
                return;
            }

            // map line item products from the cart as Google conform products
            googleProducts = cart.productLineItems.map(function(product) {
                // map price list to the correct property *price*
                var modifiedProduct = Object.assign({}, product, {price: product.listPrice}),
                    // parse relevant product data
                    googleProduct = getGoogleProduct(modifiedProduct);

                // get additional relevant product data (quantity)
                googleProduct.quantity = product.quantity;

                return googleProduct;
            });

            // generate event parameters
            eventParameters = {
                'send_to': settings.adsCampaignConversionId,
                'ecomm_pagetype': 'cart',
                'value': cart.grandTotal.amount,
                'currency': cart.productLineItems[0].listPrice.currency,
                'items': googleProducts,
                'cart_list_name': data.list ? data.list : "default_basket"
            };

            settings.debug && console.log('de_epages/googlesmartshopping/googlesmartshopping-controller - onCartViewEvent: ', cart, googleProducts, eventParameters);
            // tag google event
            gtagConfig.gtag('event', 'cart_view', eventParameters);
        },

        /*
         * Handling event if order confirmation page is shown
         */
        onOrderCompletedEvent = function(event) {
            var order = event.detail,
                googleProducts,
                eventParameters;

            if(!order || !order.lineItemContainer || order.lineItemContainer.grandTotal.amount <= 0) {
                return;
            }

            // map line item products from the order as Google conform products
            googleProducts = order.lineItemContainer.productLineItems.map(function(product) {
                // map price list to the correct property *price*
                var modifiedProduct = Object.assign({}, product, {price: product.listPrice}),
                    // parse relevant product data
                    googleProduct = getGoogleProduct(modifiedProduct);

                // get additional relevant product data (quantity)
                googleProduct.quantity = product.quantity.amount;

                return googleProduct;
            });

            // generate event parameters
            eventParameters = {
                'ecomm_pagetype': 'purchase',
                'send_to': settings.adsCampaignConversionLabel,
                'transaction_id': order.orderNumber,
                'value': order.lineItemContainer.grandTotal.amount,
                'currency': order.lineItemContainer.grandTotal.currency,
                'country': getCustomerCountryCodeFromOrder(order),
                'items': googleProducts
            };

            settings.debug && console.log('de_epages/googlesmartshopping/googlesmartshopping-controller - onOrderCompletedEvent: ', order, googleProducts, eventParameters);
            // tag google event
            gtagConfig.gtag('event', 'purchase', eventParameters);
        },

        /*
         * Parse product to get relevant data for GTAG call
         */
        getGoogleProduct = function(product) {
            return {
                'item_id': getGoogleProductId(product.sku ? product.sku : product.productNumber),
                'price': product.price ? product.price.amount : product.listPrice.amount,
                'google_business_vertical': 'retail',
                'item_name': product.name
            };
        },


        /*
         * Build Google conform ID
         */
        getGoogleProductId = function(productCode) {
            if(gtagConfig.googlePortalsLocale.length === 0) {
                return productCode;
            }

            if(gtagConfig.googlePortalsLocale.length === 1) {
                return (productCode + '_' + gtagConfig.googlePortalsLocale[0]);
            }

            var shopperLocale = getCustomerLocale(),
                matchingPortalsLocale = gtagConfig.googlePortalsLocale.filter(function(portalLocale) {
                    return portalLocale.includes(shopperLocale);
                });

            if(matchingPortalsLocale.length >= 1) {
                return (productCode + '_' + matchingPortalsLocale[0]);
            }

            return (productCode + '_' + gtagConfig.googlePortalsLocale[0]);
        },

        /*
         * Get locale of customer
         */
        getCustomerLocale = function() {
            var countryCode = '';

            if(document.documentElement && document.documentElement.lang) {
                countryCode = document.documentElement.lang;
            }

            return countryCode;
        },

        /*
         * Extract customer country code from order
         */
        getCustomerCountryCodeFromOrder = function(order) {
            var shopperCountryCode = settings.shopperCountryCode;

            if(order.shippingAddress && order.shippingAddress.country) {
                return getCountryCodeFromCountryName(order.shippingAddress.country);
            }
            if(order.billingAddress && order.billingAddress.country) {
                return getCountryCodeFromCountryName(order.billingAddress.country);
            }

            return shopperCountryCode;
        },

        /*
         * Extract country code from country name
         */
        getCountryCodeFromCountryName = function(orderCountryName) {
            var shopperCountryCode = settings.shopperCountryCode,
                availableCountryCode;

            for(availableCountryCode in gtagConfig.availableCountries) {
                if(orderCountryName.includes(gtagConfig.availableCountries[availableCountryCode])) {
                    return availableCountryCode;
                }
            }

            return shopperCountryCode;
        },

        /*
         * Initialize Google Smart Shopping controller
         */
        init = function(options) {
            var defaults = {
                //  adsCampaignConversionId,
                //  adsCampaignConversionLabel,
                //  adsCampaignRemarketingId,
                    debug: false,
                    shopperCountryCode: "de"
                };

            // extend defaults and initialize module settings
            settings = $.extend(defaults, options);

            settings.debug && console.log("settings: ", settings);
        };


    /*
     * Register events that can be fired at the appropriate places to provide
     * Google Smartshopping with the necessary data.
     */
    // to dispatch/fire event use: ep(window).trigger('product:viewlistGSS', {product: product, quantity: quantity});
    // detailed description for product, quantity, cart and so on see in comment on top of this document
    $(window).on('product:ctaGSS', function(evt, detail) {
        evt.detail = detail ? detail : undefined;
        settings.debug && console.log('product:ctaGSS: ', evt, detail);
        settings.adsCampaignConversionId && onProductCTAEvent(evt);
    });
    $(window).on('product:viewGSS', function(evt, detail) {
        evt.detail = detail ? detail : undefined;
        settings.debug && console.log('product:viewGSS: ', evt, detail);
        settings.adsCampaignConversionId && onSingleProductViewEvent(evt);
    });
    $(window).on('product:viewlistGSS', function(evt, detail) {
        evt.detail = detail ? detail : undefined;
        settings.debug && console.log('product:viewlistGSS: ', evt, detail);
        settings.adsCampaignConversionId && onProductListViewEvent(evt);
    });
    $(window).on('cart:addGSS', function(evt, detail) {
        evt.detail = detail ? detail : undefined;
        settings.debug && console.log('cart:addGSS: ', evt, detail);
        settings.adsCampaignConversionId && onCartAddEvent(evt);
    });
    $(window).on('cart:viewGSS', function(evt, detail) {
        evt.detail = detail ? detail : undefined;
        settings.adsCampaignConversionId && onCartViewEvent(evt);
        settings.debug && console.log('cart:viewGSS: ', evt, detail);
    });
    $(window).on('order:completedGSS', function(evt, detail) {
        evt.detail = detail ? detail : undefined;
        settings.adsCampaignConversionLabel && onOrderCompletedEvent(evt);
        settings.debug && console.log('order:completedGSS: ', evt, detail);
    });

    return init;
});